@@ -6,7 +6,7 @@
  *	  message integrity and endpoint authentication.
  *
  *
- * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2014, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -66,9 +66,12 @@
 #ifdef USE_SSL
 #include <openssl/ssl.h>
 #include <openssl/dh.h>
-#if OPENSSL_VERSION_NUMBER >= 0x0907000L
+#if SSLEAY_VERSION_NUMBER >= 0x0907000L
 #include <openssl/conf.h>
 #endif
+#if (OPENSSL_VERSION_NUMBER >= 0x0090800fL) && !defined(OPENSSL_NO_ECDH)
+#include <openssl/ec.h>
+#endif
 #endif   /* USE_SSL */
 
 #include "libpq/libpq.h"
@@ -80,14 +83,13 @@
 
 static DH  *load_dh_file(int keylength);
 static DH  *load_dh_buffer(const char *, size_t);
-static DH  *generate_dh_parameters(int prime_len, int generator);
 static DH  *tmp_dh_cb(SSL *s, int is_export, int keylength);
 static int	verify_cb(int, X509_STORE_CTX *);
 static void info_cb(const SSL *ssl, int type, int args);
 static void initialize_SSL(void);
 static int	open_server_SSL(Port *);
 static void close_SSL(Port *);
-static const char *SSLerrmessage(unsigned long ecode);
+static const char *SSLerrmessage(void);
 #endif
 
 char	   *ssl_cert_file;
@@ -113,6 +115,12 @@ static bool ssl_loaded_verify_locations = false;
 /* GUC variable controlling SSL cipher list */
 char	   *SSLCipherSuites = NULL;
 
+/* GUC variable for default ECHD curve. */
+char	   *SSLECDHCurve;
+
+/* GUC variable: if false, prefer client ciphers */
+bool		SSLPreferServerCiphers;
+
 /* ------------------------------------------------------------ */
 /*						 Hardcoded values						*/
 /* ------------------------------------------------------------ */
@@ -249,14 +257,11 @@ secure_read(Port *port, void *ptr, size_t len)
 	if (port->ssl)
 	{
 		int			err;
-		unsigned long ecode;
 
 rloop:
 		errno = 0;
-		ERR_clear_error();
 		n = SSL_read(port->ssl, ptr, len);
 		err = SSL_get_error(port->ssl, n);
-		ecode = (err != SSL_ERROR_NONE || n < 0) ? ERR_get_error() : 0;
 		switch (err)
 		{
 			case SSL_ERROR_NONE:
@@ -288,14 +293,12 @@ rloop:
 			case SSL_ERROR_SSL:
 				ereport(COMMERROR,
 						(errcode(ERRCODE_PROTOCOL_VIOLATION),
-						 errmsg("SSL error: %s", SSLerrmessage(ecode))));
+						 errmsg("SSL error: %s", SSLerrmessage())));
+				/* fall through */
+			case SSL_ERROR_ZERO_RETURN:
 				errno = ECONNRESET;
 				n = -1;
 				break;
-			case SSL_ERROR_ZERO_RETURN:
-				/* connection was cleanly shut down by peer */
-				n = 0;
-				break;
 			default:
 				ereport(COMMERROR,
 						(errcode(ERRCODE_PROTOCOL_VIOLATION),
@@ -331,7 +334,6 @@ secure_write(Port *port, void *ptr, size_t len)
 	if (port->ssl)
 	{
 		int			err;
-		unsigned long ecode;
 
 		/*
 		 * If SSL renegotiations are enabled and we're getting close to the
@@ -386,10 +388,8 @@ secure_write(Port *port, void *ptr, size_t len)
 
 wloop:
 		errno = 0;
-		ERR_clear_error();
 		n = SSL_write(port->ssl, ptr, len);
 		err = SSL_get_error(port->ssl, n);
-		ecode = (err != SSL_ERROR_NONE || n < 0) ? ERR_get_error() : 0;
 		switch (err)
 		{
 			case SSL_ERROR_NONE:
@@ -415,15 +415,9 @@ wloop:
 			case SSL_ERROR_SSL:
 				ereport(COMMERROR,
 						(errcode(ERRCODE_PROTOCOL_VIOLATION),
-						 errmsg("SSL error: %s", SSLerrmessage(ecode))));
-				errno = ECONNRESET;
-				n = -1;
-				break;
+						 errmsg("SSL error: %s", SSLerrmessage())));
+				/* fall through */
 			case SSL_ERROR_ZERO_RETURN:
-				/*
-				 * the SSL connnection was closed, leave it to the caller
-				 * to ereport it
-				 */
 				errno = ECONNRESET;
 				n = -1;
 				break;
@@ -485,7 +479,8 @@ wloop:
  * to retry; do we need to adopt their logic for that?
  */
 
-static BIO_METHOD *my_bio_methods = NULL;
+static bool my_bio_initialized = false;
+static BIO_METHOD my_bio_methods;
 
 static int
 my_sock_read(BIO *h, char *buf, int size)
@@ -496,7 +491,7 @@ my_sock_read(BIO *h, char *buf, int size)
 
 	if (buf != NULL)
 	{
-		res = recv(BIO_get_fd(h, NULL), buf, size, 0);
+		res = recv(h->num, buf, size, 0);
 		BIO_clear_retry_flags(h);
 		if (res <= 0)
 		{
@@ -518,7 +513,7 @@ my_sock_write(BIO *h, const char *buf, int size)
 {
 	int			res = 0;
 
-	res = send(BIO_get_fd(h, NULL), buf, size, 0);
+	res = send(h->num, buf, size, 0);
 	BIO_clear_retry_flags(h);
 	if (res <= 0)
 	{
@@ -534,41 +529,14 @@ my_sock_write(BIO *h, const char *buf, int size)
 static BIO_METHOD *
 my_BIO_s_socket(void)
 {
-	if (!my_bio_methods)
+	if (!my_bio_initialized)
 	{
-		BIO_METHOD *biom = (BIO_METHOD *) BIO_s_socket();
-#ifdef HAVE_BIO_METH_NEW
-		int			my_bio_index;
-
-		my_bio_index = BIO_get_new_index();
-		if (my_bio_index == -1)
-			return NULL;
-		my_bio_methods = BIO_meth_new(my_bio_index, "PostgreSQL backend socket");
-		if (!my_bio_methods)
-			return NULL;
-		if (!BIO_meth_set_write(my_bio_methods, my_sock_write) ||
-			!BIO_meth_set_read(my_bio_methods, my_sock_read) ||
-			!BIO_meth_set_gets(my_bio_methods, BIO_meth_get_gets(biom)) ||
-			!BIO_meth_set_puts(my_bio_methods, BIO_meth_get_puts(biom)) ||
-			!BIO_meth_set_ctrl(my_bio_methods, BIO_meth_get_ctrl(biom)) ||
-			!BIO_meth_set_create(my_bio_methods, BIO_meth_get_create(biom)) ||
-			!BIO_meth_set_destroy(my_bio_methods, BIO_meth_get_destroy(biom)) ||
-			!BIO_meth_set_callback_ctrl(my_bio_methods, BIO_meth_get_callback_ctrl(biom)))
-		{
-			BIO_meth_free(my_bio_methods);
-			my_bio_methods = NULL;
-			return NULL;
-		}
-#else
-		my_bio_methods = malloc(sizeof(BIO_METHOD));
-		if (!my_bio_methods)
-			return NULL;
-		memcpy(my_bio_methods, biom, sizeof(BIO_METHOD));
-		my_bio_methods->bread = my_sock_read;
-		my_bio_methods->bwrite = my_sock_write;
-#endif
+		memcpy(&my_bio_methods, BIO_s_socket(), sizeof(BIO_METHOD));
+		my_bio_methods.bread = my_sock_read;
+		my_bio_methods.bwrite = my_sock_write;
+		my_bio_initialized = true;
 	}
-	return my_bio_methods;
+	return &my_bio_methods;
 }
 
 /* This should exactly match openssl's SSL_set_fd except for using my BIO */
@@ -576,16 +544,9 @@ static int
 my_SSL_set_fd(SSL *s, int fd)
 {
 	int			ret = 0;
-	BIO		   *bio;
-	BIO_METHOD *bio_method;
+	BIO		   *bio = NULL;
 
-	bio_method = my_BIO_s_socket();
-	if (bio_method == NULL)
-	{
-		SSLerr(SSL_F_SSL_SET_FD, ERR_R_BUF_LIB);
-		goto err;
-	}
-	bio = BIO_new(bio_method);
+	bio = BIO_new(my_BIO_s_socket());
 
 	if (bio == NULL)
 	{
@@ -637,8 +598,7 @@ load_dh_file(int keylength)
 	{
 		if (DH_check(dh, &codes) == 0)
 		{
-			elog(LOG, "DH_check error (%s): %s", fnbuf,
-				 SSLerrmessage(ERR_get_error()));
+			elog(LOG, "DH_check error (%s): %s", fnbuf, SSLerrmessage());
 			return NULL;
 		}
 		if (codes & DH_CHECK_P_NOT_PRIME)
@@ -678,37 +638,12 @@ load_dh_buffer(const char *buffer, size_t len)
 	if (dh == NULL)
 		ereport(DEBUG2,
 				(errmsg_internal("DH load buffer: %s",
-								 SSLerrmessage(ERR_get_error()))));
+								 SSLerrmessage())));
 	BIO_free(bio);
 
 	return dh;
 }
 
-/*
- *	Generate DH parameters.
- *
- *	Last resort if we can't load precomputed nor hardcoded
- *	parameters.
- */
-static DH  *
-generate_dh_parameters(int prime_len, int generator)
-{
-#if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
-	DH		   *dh;
-
-	if ((dh = DH_new()) == NULL)
-		return NULL;
-
-	if (DH_generate_parameters_ex(dh, prime_len, generator, NULL))
-		return dh;
-
-	DH_free(dh);
-	return NULL;
-#else
-	return DH_generate_parameters(prime_len, generator, NULL, NULL);
-#endif
-}
-
 /*
  *	Generate an ephemeral DH key.  Because this can take a long
  *	time to compute, we can use precomputed parameters of the
@@ -778,7 +713,7 @@ tmp_dh_cb(SSL *s, int is_export, int keylength)
 		ereport(DEBUG2,
 				(errmsg_internal("DH: generating parameters (%d bits)",
 								 keylength)));
-		r = generate_dh_parameters(keylength, DH_GENERATOR_2);
+		r = DH_generate_parameters(keylength, DH_GENERATOR_2, NULL, NULL);
 	}
 
 	return r;
@@ -845,6 +780,31 @@ info_cb(const SSL *ssl, int type, int args)
 	}
 }
 
+#if (OPENSSL_VERSION_NUMBER >= 0x0090800fL) && !defined(OPENSSL_NO_ECDH)
+static void
+initialize_ecdh(void)
+{
+	EC_KEY	   *ecdh;
+	int			nid;
+
+	nid = OBJ_sn2nid(SSLECDHCurve);
+	if (!nid)
+		ereport(FATAL,
+				(errmsg("ECDH: unrecognized curve name: %s", SSLECDHCurve)));
+
+	ecdh = EC_KEY_new_by_curve_name(nid);
+	if (!ecdh)
+		ereport(FATAL,
+				(errmsg("ECDH: could not create key")));
+
+	SSL_CTX_set_options(SSL_context, SSL_OP_SINGLE_ECDH_USE);
+	SSL_CTX_set_tmp_ecdh(SSL_context, ecdh);
+	EC_KEY_free(ecdh);
+}
+#else
+#define initialize_ecdh()
+#endif
+
 /*
  *	Initialize global SSL context.
  */
@@ -857,27 +817,23 @@ initialize_SSL(void)
 
 	if (!SSL_context)
 	{
-#ifdef HAVE_OPENSSL_INIT_SSL
-		OPENSSL_init_ssl(OPENSSL_INIT_LOAD_CONFIG, NULL);
-#else
 #if SSLEAY_VERSION_NUMBER >= 0x0907000L
 		OPENSSL_config(NULL);
 #endif
 		SSL_library_init();
 		SSL_load_error_strings();
-#endif
 
 		/*
 		 * We use SSLv23_method() because it can negotiate use of the highest
 		 * mutually supported protocol version, while alternatives like
 		 * TLSv1_2_method() permit only one specific version.  Note that we
-		 * don't actually allow SSL v2, only v3 and TLS protocols (see below).
+		 * don't actually allow SSL v2 or v3, only TLS protocols (see below).
 		 */
 		SSL_context = SSL_CTX_new(SSLv23_method());
 		if (!SSL_context)
 			ereport(FATAL,
 					(errmsg("could not create SSL context: %s",
-							SSLerrmessage(ERR_get_error()))));
+							SSLerrmessage())));
 
 		/*
 		 * Disable OpenSSL's moving-write-buffer sanity check, because it
@@ -893,7 +849,7 @@ initialize_SSL(void)
 			ereport(FATAL,
 					(errcode(ERRCODE_CONFIG_FILE_ERROR),
 				  errmsg("could not load server certificate file \"%s\": %s",
-						 ssl_cert_file, SSLerrmessage(ERR_get_error()))));
+						 ssl_cert_file, SSLerrmessage())));
 
 		if (stat(ssl_key_file, &buf) != 0)
 			ereport(FATAL,
@@ -923,30 +879,31 @@ initialize_SSL(void)
 										SSL_FILETYPE_PEM) != 1)
 			ereport(FATAL,
 					(errmsg("could not load private key file \"%s\": %s",
-							ssl_key_file, SSLerrmessage(ERR_get_error()))));
+							ssl_key_file, SSLerrmessage())));
 
 		if (SSL_CTX_check_private_key(SSL_context) != 1)
 			ereport(FATAL,
 					(errmsg("check of private key failed: %s",
-							SSLerrmessage(ERR_get_error()))));
+							SSLerrmessage())));
 	}
 
-	/* set up ephemeral DH keys, and disallow SSL v2 while at it */
+	/* set up ephemeral DH keys, and disallow SSL v2/v3 while at it */
 	SSL_CTX_set_tmp_dh_callback(SSL_context, tmp_dh_cb);
-	SSL_CTX_set_options(SSL_context, SSL_OP_SINGLE_DH_USE | SSL_OP_NO_SSLv2);
+	SSL_CTX_set_options(SSL_context,
+						SSL_OP_SINGLE_DH_USE |
+						SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3);
 
-	/* disallow SSL session tickets */
-#ifdef SSL_OP_NO_TICKET			/* added in openssl 0.9.8f */
-	SSL_CTX_set_options(SSL_context, SSL_OP_NO_TICKET);
-#endif
-
-	/* disallow SSL session caching, too */
-	SSL_CTX_set_session_cache_mode(SSL_context, SSL_SESS_CACHE_OFF);
+	/* set up ephemeral ECDH keys */
+	initialize_ecdh();
 
 	/* set up the allowed cipher list */
 	if (SSL_CTX_set_cipher_list(SSL_context, SSLCipherSuites) != 1)
 		elog(FATAL, "could not set the cipher list (no valid ciphers available)");
 
+	/* Let server choose order */
+	if (SSLPreferServerCiphers)
+		SSL_CTX_set_options(SSL_context, SSL_OP_CIPHER_SERVER_PREFERENCE);
+
 	/*
 	 * Load CA store, so we can verify client certificates if needed.
 	 */
@@ -956,7 +913,7 @@ initialize_SSL(void)
 			(root_cert_list = SSL_load_client_CA_file(ssl_ca_file)) == NULL)
 			ereport(FATAL,
 					(errmsg("could not load root certificate file \"%s\": %s",
-							ssl_ca_file, SSLerrmessage(ERR_get_error()))));
+							ssl_ca_file, SSLerrmessage())));
 	}
 
 	/*----------
@@ -987,7 +944,7 @@ initialize_SSL(void)
 			else
 				ereport(FATAL,
 						(errmsg("could not load SSL certificate revocation list file \"%s\": %s",
-								ssl_crl_file, SSLerrmessage(ERR_get_error()))));
+								ssl_crl_file, SSLerrmessage())));
 		}
 	}
 
@@ -1023,7 +980,6 @@ open_server_SSL(Port *port)
 {
 	int			r;
 	int			err;
-	unsigned long ecode;
 
 	Assert(!port->ssl);
 	Assert(!port->peer);
@@ -1033,7 +989,7 @@ open_server_SSL(Port *port)
 		ereport(COMMERROR,
 				(errcode(ERRCODE_PROTOCOL_VIOLATION),
 				 errmsg("could not initialize SSL connection: %s",
-						SSLerrmessage(ERR_get_error()))));
+						SSLerrmessage())));
 		return -1;
 	}
 	if (!my_SSL_set_fd(port->ssl, port->sock))
@@ -1041,35 +997,15 @@ open_server_SSL(Port *port)
 		ereport(COMMERROR,
 				(errcode(ERRCODE_PROTOCOL_VIOLATION),
 				 errmsg("could not set SSL socket: %s",
-						SSLerrmessage(ERR_get_error()))));
+						SSLerrmessage())));
 		return -1;
 	}
 
 aloop:
-	/*
-	 * Prepare to call SSL_get_error() by clearing thread's OpenSSL error
-	 * queue.  In general, the current thread's error queue must be empty
-	 * before the TLS/SSL I/O operation is attempted, or SSL_get_error()
-	 * will not work reliably.  An extension may have failed to clear the
-	 * per-thread error queue following another call to an OpenSSL I/O
-	 * routine.
-	 */
-	ERR_clear_error();
 	r = SSL_accept(port->ssl);
 	if (r <= 0)
 	{
 		err = SSL_get_error(port->ssl, r);
-
-		/*
-		 * Other clients of OpenSSL in the backend may fail to call
-		 * ERR_get_error(), but we always do, so as to not cause problems
-		 * for OpenSSL clients that don't call ERR_clear_error()
-		 * defensively.  Be sure that this happens by calling now.
-		 * SSL_get_error() relies on the OpenSSL per-thread error queue
-		 * being intact, so this is the earliest possible point
-		 * ERR_get_error() may be called.
-		 */
-		ecode = ERR_get_error();
 		switch (err)
 		{
 			case SSL_ERROR_WANT_READ:
@@ -1095,7 +1031,7 @@ aloop:
 				ereport(COMMERROR,
 						(errcode(ERRCODE_PROTOCOL_VIOLATION),
 						 errmsg("could not accept SSL connection: %s",
-								SSLerrmessage(ecode))));
+								SSLerrmessage())));
 				break;
 			case SSL_ERROR_ZERO_RETURN:
 				ereport(COMMERROR,
@@ -1196,24 +1132,24 @@ close_SSL(Port *port)
 /*
  * Obtain reason string for last SSL error
  *
- * ERR_get_error() is used by caller to get errcode to pass here.
- *
  * Some caution is needed here since ERR_reason_error_string will
  * return NULL if it doesn't recognize the error code.  We don't
  * want to return NULL ever.
  */
 static const char *
-SSLerrmessage(unsigned long ecode)
+SSLerrmessage(void)
 {
+	unsigned long errcode;
 	const char *errreason;
 	static char errbuf[32];
 
-	if (ecode == 0)
+	errcode = ERR_get_error();
+	if (errcode == 0)
 		return _("no SSL error reported");
-	errreason = ERR_reason_error_string(ecode);
+	errreason = ERR_reason_error_string(errcode);
 	if (errreason != NULL)
 		return errreason;
-	snprintf(errbuf, sizeof(errbuf), _("SSL error code %lu"), ecode);
+	snprintf(errbuf, sizeof(errbuf), _("SSL error code %lu"), errcode);
 	return errbuf;
 }
 
